@@ -62,11 +62,16 @@ Common options:
                                appear as the header row in the output.
     -d, --delimiter <arg>      The field delimiter for reading CSV data.
                                Must be a single character. [default: ,]
-    --no-memcheck              Do not check if there is enough memory to load the
-                               entire CSV into memory.
+    --memcheck                 Check if there is enough memory to load the entire
+                               CSV into memory using CONSERVATIVE heuristics.
 "#;
 
-use std::{collections::HashSet, fs::File, io::Write, path::Path};
+use std::{
+    collections::HashSet,
+    fs::File,
+    io::{BufReader, Write},
+    path::Path,
+};
 
 use ahash::AHashMap;
 use csv::ByteRecord;
@@ -79,7 +84,7 @@ use stats::Frequencies;
 
 use crate::{
     cmd::stats::Stats,
-    config::{Config, Delimiter},
+    config::{Config, Delimiter, DEFAULT_RDR_BUFFER_CAPACITY},
     select::SelectColumns,
     util, CliResult,
 };
@@ -96,7 +101,7 @@ pub struct Args {
     pub flag_no_headers:      bool,
     pub flag_delimiter:       Option<Delimiter>,
     pub arg_input:            Option<String>,
-    pub flag_no_memcheck:     bool,
+    pub flag_memcheck:        bool,
 }
 
 const STDIN_CSV: &str = "stdin.csv";
@@ -130,12 +135,12 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     util::mem_file_check(
         &std::path::PathBuf::from(&input_path),
         false,
-        args.flag_no_memcheck,
+        args.flag_memcheck,
     )?;
 
     // we can do this directly here, since args is mutable and
     // Config has not been created yet at this point
-    args.flag_prefer_dmy = args.flag_prefer_dmy || std::env::var("QSV_PREFER_DMY").is_ok();
+    args.flag_prefer_dmy = args.flag_prefer_dmy || util::get_envvar_flag("QSV_PREFER_DMY");
     if args.flag_prefer_dmy {
         winfo!("Prefer DMY set.");
     }
@@ -429,13 +434,12 @@ fn get_stats_records(args: &Args) -> CliResult<(ByteRecord, Vec<Stats>, AHashMap
         flag_output:          None,
         flag_no_headers:      args.flag_no_headers,
         flag_delimiter:       args.flag_delimiter,
-        flag_no_memcheck:     args.flag_no_memcheck,
+        flag_memcheck:        args.flag_memcheck,
         flag_stats_binout:    None,
     };
 
-    let stats_binary_encoded_path = args.arg_input.clone().unwrap();
-    let stats_binary_encoded_path = Path::new(&stats_binary_encoded_path)
-        .to_path_buf()
+    let stats_binary_encoded_path = Path::new(&args.arg_input.clone().unwrap())
+        .canonicalize()?
         .with_extension("stats.csv.bin");
 
     let stats_bin_current = if stats_binary_encoded_path.exists() {
@@ -458,10 +462,13 @@ fn get_stats_records(args: &Args) -> CliResult<(ByteRecord, Vec<Stats>, AHashMap
     let mut stats_bin_loaded = false;
 
     // if stats.bin file exists and is current, use it
-    let mut csv_stats: Vec<Stats> = Vec::with_capacity(100_000_000);
+    let mut csv_stats: Vec<Stats> = Vec::new();
 
     if stats_bin_current {
-        let mut bin_file = File::open(stats_binary_encoded_path)?;
+        let mut bin_file = BufReader::with_capacity(
+            DEFAULT_RDR_BUFFER_CAPACITY * 4,
+            File::open(stats_binary_encoded_path)?,
+        );
         match bincode::deserialize_from(&mut bin_file) {
             Ok(stats) => {
                 csv_stats = stats;
@@ -511,8 +518,8 @@ fn get_stats_records(args: &Args) -> CliResult<(ByteRecord, Vec<Stats>, AHashMap
             let delim = delimiter.as_byte() as char;
             stats_args_str = format!("{stats_args_str} --delimiter {delim}");
         }
-        if args.flag_no_memcheck {
-            stats_args_str = format!("{stats_args_str} --no-memcheck");
+        if args.flag_memcheck {
+            stats_args_str = format!("{stats_args_str} --memcheck");
         }
         if let Some(mut jobs) = stats_args.flag_jobs {
             if jobs > 2 {
@@ -528,12 +535,21 @@ fn get_stats_records(args: &Args) -> CliResult<(ByteRecord, Vec<Stats>, AHashMap
         stats_cmd.args(stats_args_vec);
         let _stats_output = stats_cmd.output()?;
 
-        let mut bin_file = File::open(tempfile_statsbin_path).unwrap();
+        let mut bin_file = BufReader::with_capacity(
+            DEFAULT_RDR_BUFFER_CAPACITY,
+            File::open(tempfile_statsbin_path).unwrap(),
+        );
 
-        #[allow(unused_assignments)]
-        let mut decoded_stats: Vec<Stats> = Vec::with_capacity(100_000_000); // minimize allocs
-        decoded_stats = bincode::deserialize_from(&mut bin_file).unwrap();
-        csv_stats = decoded_stats;
+        match bincode::deserialize_from(&mut bin_file) {
+            Ok(stats) => {
+                csv_stats = stats;
+            }
+            Err(e) => {
+                return fail_clierror!(
+                    "Error reading stats.csv.bin file: {e:?}. Schema generation aborted."
+                );
+            }
+        }
     };
 
     // get the headers from the input file
@@ -601,16 +617,16 @@ fn get_unique_values(
 ) -> CliResult<AHashMap<String, Vec<String>>> {
     // prepare arg for invoking cmd::frequency
     let freq_args = crate::cmd::frequency::Args {
-        arg_input:        args.arg_input.clone(),
-        flag_select:      crate::select::SelectColumns::parse(column_select_arg).unwrap(),
-        flag_limit:       args.flag_enum_threshold,
-        flag_asc:         false,
-        flag_no_nulls:    true,
-        flag_jobs:        Some(util::njobs(args.flag_jobs)),
-        flag_output:      None,
-        flag_no_headers:  args.flag_no_headers,
-        flag_delimiter:   args.flag_delimiter,
-        flag_no_memcheck: args.flag_no_memcheck,
+        arg_input:       args.arg_input.clone(),
+        flag_select:     crate::select::SelectColumns::parse(column_select_arg).unwrap(),
+        flag_limit:      args.flag_enum_threshold,
+        flag_asc:        false,
+        flag_no_nulls:   true,
+        flag_jobs:       Some(util::njobs(args.flag_jobs)),
+        flag_output:     None,
+        flag_no_headers: args.flag_no_headers,
+        flag_delimiter:  args.flag_delimiter,
+        flag_memcheck:   args.flag_memcheck,
     };
 
     let (headers, ftables) = match freq_args.rconfig().indexed()? {
