@@ -130,8 +130,8 @@ Common options:
                            in statistics.
     -d, --delimiter <arg>  The field delimiter for reading CSV data.
                            Must be a single character. (default: ,)
-    --no-memcheck          Do not check if there is enough memory to load the
-                           entire CSV into memory.
+    --memcheck             Check if there is enough memory to load the entire
+                           CSV into memory using CONSERVATIVE heuristics.
 "#;
 
 /*
@@ -156,7 +156,7 @@ use std::{
     borrow::ToOwned,
     default::Default,
     fmt, fs, io,
-    io::Write,
+    io::{BufWriter, Write},
     iter::repeat,
     path::{Path, PathBuf},
     str,
@@ -174,7 +174,7 @@ use threadpool::ThreadPool;
 
 use self::FieldType::{TDate, TDateTime, TFloat, TInteger, TNull, TString};
 use crate::{
-    config::{Config, Delimiter},
+    config::{Config, Delimiter, DEFAULT_WTR_BUFFER_CAPACITY},
     select::{SelectColumns, Selection},
     util, CliResult,
 };
@@ -201,7 +201,7 @@ pub struct Args {
     pub flag_output:          Option<String>,
     pub flag_no_headers:      bool,
     pub flag_delimiter:       Option<Delimiter>,
-    pub flag_no_memcheck:     bool,
+    pub flag_memcheck:        bool,
     pub flag_stats_binout:    Option<String>,
 }
 
@@ -227,6 +227,11 @@ struct StatsArgs {
     flag_no_headers:      bool,
     flag_delimiter:       String,
     flag_output_snappy:   bool,
+    canonical_input_path: String,
+    canonical_stats_path: String,
+    record_count:         u64,
+    date_generated:       String,
+    compute_duration_ms:  u64,
 }
 
 static INFER_DATE_FLAGS: once_cell::sync::OnceCell<Vec<bool>> = OnceCell::new();
@@ -255,7 +260,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     // save the current args, we'll use it to generate
     // the stats.csv.json file
-    let current_stats_args = StatsArgs {
+    let mut current_stats_args = StatsArgs {
         arg_input:            format!("{:?}", args.arg_input),
         flag_select:          format!("{:?}", args.flag_select),
         flag_everything:      args.flag_everything,
@@ -281,6 +286,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             let p = args.flag_output.clone().unwrap();
             p.to_lowercase().ends_with(".sz")
         },
+        canonical_input_path: String::new(),
+        canonical_stats_path: String::new(),
+        record_count:         0,
+        date_generated:       String::new(),
+        compute_duration_ms:  0,
     };
 
     // create a temporary file to store the <FILESTEM>.stats.csv file
@@ -316,21 +326,28 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 .unwrap(),
         );
         fconfig.path = Some(tempfile_path);
+    } else {
+        // check if the input file exists
+        if let Some(path) = fconfig.path.clone() {
+            if !path.exists() {
+                return fail_clierror!("File {:?} does not exist", path.display());
+            }
+        }
     }
 
-    // create stats_binary_encoded_vec to store the stats in binary format
-    let mut stats_binary_encoded_vec: Vec<u8> = Vec::with_capacity(100_000_000);
+    // create stats_for_encoding to store the stats in binary format
+    let mut stats_for_encoding: Vec<Stats> = Vec::new();
 
     let mut compute_stats = true;
 
     if let Some(path) = fconfig.path.clone() {
         let path_file_stem = path.file_stem().unwrap().to_str().unwrap();
         let stats_file = stats_path(&path, false);
-        // check if <FILESTEM>.stats.csv file already exists and
-        // if it does, check if it was compiled using the same args.
-        // However, if the --force flag is set, we will regenerate the stats
+        // check if <FILESTEM>.stats.csv file already exists.
+        // If it does, check if it was compiled using the same args.
+        // However, if the --force flag is set,
+        // regenerate the stats even if the args are the same.
         if stats_file.exists() && !args.flag_force {
-            // check if the existing stats were compiled using the same args
             let stats_args_json_file = stats_file.with_extension("csv.json");
             let existing_stats_args_json_str =
                 match fs::read_to_string(stats_args_json_file.clone()) {
@@ -348,8 +365,17 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
             // deserialize the existing stats args json
             let existing_stats_args_json: StatsArgs =
-                match serde_json::from_str(&existing_stats_args_json_str) {
-                    Ok(stat_args) => stat_args,
+                match serde_json::from_str::<StatsArgs>(&existing_stats_args_json_str) {
+                    Ok(mut stat_args) => {
+                        // we init these fields to empty values because we don't want to compare
+                        // them when checking if the args are the same
+                        stat_args.canonical_input_path = String::new();
+                        stat_args.canonical_stats_path = String::new();
+                        stat_args.record_count = 0;
+                        stat_args.date_generated = String::new();
+                        stat_args.compute_duration_ms = 0;
+                        stat_args
+                    }
                     Err(e) => {
                         log::warn!(
                             "Could not serialize {path_file_stem}.stats.csv.json: {e:?}, \
@@ -361,9 +387,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     }
                 };
 
-            // check if the cached stats are current, use the same args or if the --everything flag
-            // was set & all the other non-stats args are equal. If so, we don't need to recompute
-            // the stats
+            // check if the cached stats are current (ie the stats file is newer than the input
+            // file), use the same args or if the --everything flag was set, and
+            // all the other non-stats args are equal. If so, we don't need to recompute the stats
             let input_file_modified = fs::metadata(&path)?.modified()?;
             let stats_file_modified = fs::metadata(&stats_file)?.modified()?;
             #[allow(clippy::nonminimal_bool)]
@@ -395,6 +421,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             }
         }
         if compute_stats {
+            let start_time = std::time::Instant::now();
+
             // we're loading the entire file into memory, we need to check avail mem
             if args.flag_everything
                 || args.flag_mode
@@ -403,9 +431,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 || args.flag_quartiles
                 || args.flag_mad
             {
-                util::mem_file_check(&path, false, args.flag_no_memcheck)?;
+                util::mem_file_check(&path, false, args.flag_memcheck)?;
             }
 
+            // we need to count the number of records in the file to calculate sparsity
             let record_count = RECORD_COUNT.get_or_init(|| util::count_rows(&fconfig).unwrap());
 
             log::info!("scanning {record_count} records...");
@@ -425,8 +454,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 }
             }?;
 
-            // store binary version of the stats in memory, so we can write it to disk later
-            stats_binary_encoded_vec = bincode::serialize(&stats).unwrap();
+            // clone a copy of stats so we can binary encode it to disk later
+            stats_for_encoding = stats.clone();
 
             let stats = args.stats_to_records(stats);
 
@@ -441,15 +470,22 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 let stat = stat.iter().map(str::as_bytes);
                 wtr.write_record(vec![&*header].into_iter().chain(stat))?;
             }
+
+            // update the stats args json file
+            current_stats_args.canonical_input_path =
+                path.canonicalize()?.to_str().unwrap().to_string();
+            current_stats_args.record_count = *record_count;
+            current_stats_args.compute_duration_ms = start_time.elapsed().as_millis() as u64;
+            current_stats_args.date_generated = chrono::Utc::now().to_rfc3339();
         }
     }
 
     wtr.flush()?;
 
-    if stdin_tempfile_path.is_some() {
+    if let Some(pb) = stdin_tempfile_path {
         // remove the temp file we created to store stdin
         log::info!("deleting stdin temp file");
-        std::fs::remove_file(stdin_tempfile_path.unwrap())?;
+        std::fs::remove_file(pb)?;
     }
 
     let currstats_filename = if compute_stats {
@@ -476,28 +512,58 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         )?;
     } else if let Some(path) = fconfig.path {
         // if we read from a file, copy the temp stats file to "<FILESTEM>.stats.csv"
-        let mut stats_pathbuf = path;
+        let mut stats_pathbuf = path.clone();
         stats_pathbuf.set_extension("stats.csv");
         if currstats_filename != stats_pathbuf.to_str().unwrap() {
             // if the stats file is not the same as the input file, copy it
             fs::copy(currstats_filename.clone(), stats_pathbuf.clone())?;
         }
 
-        // save the stats args to "<FILESTEM>.stats.csv.json"
-        stats_pathbuf.set_extension("csv.json");
-        std::fs::write(
-            stats_pathbuf.clone(),
-            serde_json::to_string_pretty(&current_stats_args).unwrap(),
-        )?;
+        // only create the stats args json and binary encoded files if we computed the stats
+        if compute_stats {
+            // save the stats args to "<FILESTEM>.stats.csv.json"
+            stats_pathbuf.set_extension("csv.json");
+            // write empty file first so we can canonicalize it
+            std::fs::File::create(stats_pathbuf.clone())?;
+            current_stats_args.canonical_stats_path = stats_pathbuf
+                .clone()
+                .canonicalize()?
+                .to_str()
+                .unwrap()
+                .to_string();
+            std::fs::write(
+                stats_pathbuf.clone(),
+                // safety: we know that current_stats_args is JSON serializable
+                serde_json::to_string_pretty(&current_stats_args).unwrap(),
+            )?;
 
-        // save the binary encoded stats to "<FILESTEM>.stats.csv.bin"
-        stats_pathbuf.set_extension("bin");
-        if stats_binary_encoded_vec.len() > 0 {
-            fs::write(stats_pathbuf.clone(), stats_binary_encoded_vec)?;
+            // binary encode the stats to "<FILESTEM>.stats.csv.bin"
+            let mut stats_pathbuf = path.clone();
+            stats_pathbuf.set_extension("stats.csv.bin");
+            // we do the binary encoding inside a block so that the encoded_file
+            // gets dropped/flushed before we copy it to the output file
+            {
+                let encoded_file = BufWriter::with_capacity(
+                    DEFAULT_WTR_BUFFER_CAPACITY * 2,
+                    fs::File::create(stats_pathbuf.clone())?,
+                );
+                if let Err(e) = bincode::serialize_into(encoded_file, &stats_for_encoding) {
+                    return fail_clierror!(
+                        "Failed to write binary encoded stats {}: {e:?}",
+                        stats_pathbuf.display()
+                    );
+                }
+            }
+        }
 
-            // if the user specified --stats-binout, copy the binary stats to that file as well
-            if let Some(stats_binout) = args.flag_stats_binout {
-                fs::copy(stats_pathbuf, stats_binout)?;
+        // if the user specified --stats-binout, copy the binary stats to that file as well
+        if let Some(stats_binout) = args.flag_stats_binout {
+            let mut stats_pathbuf = path;
+            stats_pathbuf.set_extension("stats.csv.bin");
+            if let Err(e) = fs::copy(stats_pathbuf, stats_binout.clone()) {
+                return fail_clierror!(
+                    "Failed to copy binary encoded stats to {stats_binout}: {e:?}"
+                );
             }
         }
     }
@@ -709,17 +775,21 @@ impl Args {
             fields.push("cardinality");
         }
         if self.flag_mode || all {
-            fields.push("mode");
-            fields.push("mode_count");
-            fields.push("mode_occurrences");
-            fields.push("antimode");
-            fields.push("antimode_count");
-            fields.push("antimode_occurrences");
+            fields.extend_from_slice(&[
+                "mode",
+                "mode_count",
+                "mode_occurrences",
+                "antimode",
+                "antimode_count",
+                "antimode_occurrences",
+            ]);
         }
         csv::StringRecord::from(fields)
     }
 }
 
+// returns the path to the stats file
+// safety: unwraps are safe because we know stats_csv_path is a valid path
 fn stats_path(stats_csv_path: &Path, stdin_flag: bool) -> PathBuf {
     let mut p = stats_csv_path
         .to_path_buf()
@@ -762,7 +832,7 @@ fn init_date_inference(
     flag_whitelist: &str,
 ) -> Result<(), String> {
     if infer_dates {
-        let dmy_preferred = prefer_dmy || std::env::var("QSV_PREFER_DMY").is_ok();
+        let dmy_preferred = prefer_dmy || util::get_envvar_flag("QSV_PREFER_DMY");
         DMY_PREFERENCE.store(dmy_preferred, Ordering::Relaxed);
 
         let whitelist_lower = flag_whitelist.to_lowercase();
@@ -840,6 +910,7 @@ pub struct Stats {
     which:     WhichStats,
 }
 
+#[inline]
 fn timestamp_ms_to_rfc3339(timestamp: i64, typ: FieldType) -> String {
     use chrono::{DateTime, NaiveDateTime, Utc};
 
@@ -1024,34 +1095,25 @@ impl Stats {
             .as_ref()
             .and_then(|mm| mm.show(typ, round_places))
         {
-            pieces.push(mm.0);
-            pieces.push(mm.1);
-            pieces.push(mm.2);
+            pieces.extend_from_slice(&[mm.0, mm.1, mm.2]);
         } else {
-            pieces.push(empty());
-            pieces.push(empty());
-            pieces.push(empty());
+            pieces.extend_from_slice(&[empty(), empty(), empty()]);
         }
 
         // min/max length
         if typ == FieldType::TDate || typ == FieldType::TDateTime {
             // returning min/max length for dates doesn't make sense
             // especially since we convert the date stats to rfc3339 format
-            pieces.push(empty());
-            pieces.push(empty());
+            pieces.extend_from_slice(&[empty(), empty()]);
         } else if let Some(mm) = self.minmax.as_ref().and_then(TypedMinMax::len_range) {
-            pieces.push(mm.0);
-            pieces.push(mm.1);
+            pieces.extend_from_slice(&[mm.0, mm.1]);
         } else {
-            pieces.push(empty());
-            pieces.push(empty());
+            pieces.extend_from_slice(&[empty(), empty()]);
         }
 
         // mean, stddev & variance
         if typ == TString || typ == TNull {
-            pieces.push(empty());
-            pieces.push(empty());
-            pieces.push(empty());
+            pieces.extend_from_slice(&[empty(), empty(), empty()]);
         } else if let Some(ref v) = self.online {
             if self.typ == TFloat || self.typ == TInteger {
                 pieces.push(util::round_num(v.mean(), round_places));
@@ -1075,9 +1137,7 @@ impl Stats {
                 pieces.push(empty());
             }
         } else {
-            pieces.push(empty());
-            pieces.push(empty());
-            pieces.push(empty());
+            pieces.extend_from_slice(&[empty(), empty(), empty()]);
         }
 
         // nullcount
@@ -1140,15 +1200,17 @@ impl Stats {
         }) {
             None => {
                 if self.which.quartiles {
-                    pieces.push(empty());
-                    pieces.push(empty());
-                    pieces.push(empty());
-                    pieces.push(empty());
-                    pieces.push(empty());
-                    pieces.push(empty());
-                    pieces.push(empty());
-                    pieces.push(empty());
-                    pieces.push(empty());
+                    pieces.extend_from_slice(&[
+                        empty(),
+                        empty(),
+                        empty(),
+                        empty(),
+                        empty(),
+                        empty(),
+                        empty(),
+                        empty(),
+                        empty(),
+                    ]);
                 }
             }
             Some((q1, q2, q3)) => {
@@ -1219,10 +1281,7 @@ impl Stats {
                     pieces.push(empty());
                 }
                 if self.which.mode {
-                    pieces.push(empty());
-                    pieces.push(empty());
-                    pieces.push(empty());
-                    pieces.push(empty());
+                    pieces.extend_from_slice(&[empty(), empty(), empty(), empty()]);
                 }
             }
             Some(ref mut v) => {
@@ -1334,7 +1393,11 @@ impl FieldType {
             return (FieldType::TString, None);
         }
 
-        let string = from_utf8(sample).unwrap();
+        let Ok(string) = from_utf8(sample) else {
+            // if the string is not valid utf8, we assume it is a binary string
+            // and return a string type
+            return (FieldType::TString, None);
+        };
 
         if current_type == FieldType::TFloat
             || current_type == FieldType::TInteger
@@ -1626,7 +1689,7 @@ impl Commute for TypedMinMax {
 #[allow(clippy::inline_always)]
 #[inline(always)]
 fn from_bytes<T: std::str::FromStr>(bytes: &[u8]) -> Option<T> {
-    if let Ok(x) = from_utf8(bytes) {
+    if let Ok(x) = simdutf8::basic::from_utf8(bytes) {
         x.parse().ok()
     } else {
         None
