@@ -41,10 +41,12 @@ pub fn num_cpus() -> usize {
 }
 
 pub static DEFAULT_USER_AGENT: &str = concat!(
-    env!("CARGO_PKG_NAME"),
+    env!("CARGO_BIN_NAME"),
     "/",
     env!("CARGO_PKG_VERSION"),
-    " (https://github.com/jqnatividad/qsv)",
+    " (",
+    env!("TARGET"),
+    "; https://github.com/jqnatividad/qsv)",
 );
 const TARGET: &str = match option_env!("TARGET") {
     Some(target) => target,
@@ -98,6 +100,25 @@ pub fn timeout_secs(timeout: u16) -> Result<u64, String> {
     Ok(timeout as u64)
 }
 
+/// sets user agent
+/// if user agent is not set, then use the default user agent
+pub fn set_user_agent(user_agent: Option<String>) -> CliResult<String> {
+    use reqwest::header::HeaderValue;
+
+    let ua = match user_agent {
+        Some(ua_arg) => ua_arg,
+        None => env::var("QSV_USER_AGENT").unwrap_or_else(|_| DEFAULT_USER_AGENT.to_string()),
+    };
+
+    match HeaderValue::from_str(ua.as_str()) {
+        Ok(_) => (),
+        Err(e) => return fail_clierror!("Invalid user-agent value: {e}"),
+    };
+
+    log::info!("set user agent: {ua}");
+    Ok(ua)
+}
+
 pub fn version() -> String {
     let mut enabled_features = String::new();
 
@@ -391,7 +412,17 @@ pub fn file_metadata(md: &fs::Metadata) -> (u64, u64) {
     (last_modified, fsize)
 }
 
-pub fn mem_file_check(path: &Path, version_check: bool, no_memcheck: bool) -> Result<i64, String> {
+/// Check if there is enough memory to process the file.
+/// Return the maximum file size that can be processed.
+/// If the file is larger than the maximum file size, return an error.
+/// If memcheck is true, check memory in CONSERVATIVE mode (i.e., Filesize < AVAIL memory + SWAP -
+/// headroom) If memcheck is false, check memory in NORMAL mode (i.e., Filesize < TOTAL memory -
+/// headroom)
+pub fn mem_file_check(
+    path: &Path,
+    version_check: bool,
+    conservative_memcheck: bool,
+) -> CliResult<i64> {
     // if we're NOT calling this from the version() and the file doesn't exist,
     // we don't need to check memory as file existence is checked before this function is called.
     // If we do get here with a non-existent file, that means we're using stdin,
@@ -400,7 +431,7 @@ pub fn mem_file_check(path: &Path, version_check: bool, no_memcheck: bool) -> Re
         return Ok(-1_i64);
     }
 
-    let no_memcheck_work = env::var("QSV_NO_MEMORY_CHECK").is_ok() || no_memcheck;
+    let conservative_memcheck_work = get_envvar_flag("QSV_MEMORY_CHECK") || conservative_memcheck;
 
     let mut sys = sysinfo::System::new();
     sys.refresh_memory();
@@ -417,10 +448,10 @@ pub fn mem_file_check(path: &Path, version_check: bool, no_memcheck: bool) -> Re
     mem_pct = mem_pct.clamp(10, 90);
 
     #[allow(clippy::cast_precision_loss)]
-    let max_avail_mem = if no_memcheck_work {
-        (total_mem as f32 * ((100 - mem_pct) as f32 / 100.0_f32)) as u64
-    } else {
+    let max_avail_mem = if conservative_memcheck_work {
         ((avail_mem + free_swap) as f32 * ((100 - mem_pct) as f32 / 100.0_f32)) as u64
+    } else {
+        (total_mem as f32 * ((100 - mem_pct) as f32 / 100.0_f32)) as u64
     };
 
     // if we're calling this from version(), we don't need to check the file size
@@ -428,22 +459,24 @@ pub fn mem_file_check(path: &Path, version_check: bool, no_memcheck: bool) -> Re
         let file_metadata =
             fs::metadata(path).map_err(|e| format!("Failed to get file size: {e}"))?;
         let fsize = file_metadata.len();
-        let detail_msg = format!(
-            "qsv running in non-streaming mode. Total memory: {total_mem} Available memory: \
-             {avail_mem}. Free swap: {free_swap} Max Available memory/Max input file size: \
-             {max_avail_mem}. QSV_FREEMEMORY_HEADROOM_PCT: {mem_pct}%. File size: {fsize}.",
-            total_mem = indicatif::HumanBytes(total_mem),
-            avail_mem = indicatif::HumanBytes(avail_mem),
-            free_swap = indicatif::HumanBytes(free_swap),
-            max_avail_mem = indicatif::HumanBytes(max_avail_mem),
-            mem_pct = mem_pct,
-            fsize = indicatif::HumanBytes(fsize)
-        );
-        log::info!("{detail_msg}");
         if fsize > max_avail_mem {
-            return fail!(format!(
-                "Not enough memory to process the file. {detail_msg}"
-            ));
+            return fail_clierror!(
+                "Not enough memory to process the file. qsv running in non-streaming {mode} mode. \
+                 Total memory: {total_mem} Available memory: {avail_mem}. Free swap: {free_swap} \
+                 Max Available memory/Max input file size: {max_avail_mem}. \
+                 QSV_FREEMEMORY_HEADROOM_PCT: {mem_pct}%. File size: {fsize}.",
+                mode = if conservative_memcheck_work {
+                    "CONSERVATIVE"
+                } else {
+                    "NORMAL"
+                },
+                total_mem = indicatif::HumanBytes(total_mem),
+                avail_mem = indicatif::HumanBytes(avail_mem),
+                free_swap = indicatif::HumanBytes(free_swap),
+                max_avail_mem = indicatif::HumanBytes(max_avail_mem),
+                mem_pct = mem_pct,
+                fsize = indicatif::HumanBytes(fsize)
+            );
         }
     }
 
@@ -605,7 +638,7 @@ pub fn init_logger() -> (String, flexi_logger::LoggerHandle) {
 
     let qsv_log_env = env::var("QSV_LOG_LEVEL").unwrap_or_else(|_| "off".to_string());
     let qsv_log_dir = env::var("QSV_LOG_DIR").unwrap_or_else(|_| ".".to_string());
-    let write_mode = if env::var("QSV_LOG_UNBUFFERED").is_ok() {
+    let write_mode = if get_envvar_flag("QSV_LOG_UNBUFFERED") {
         flexi_logger::WriteMode::Direct
     } else {
         flexi_logger::WriteMode::BufferAndFlush
@@ -645,7 +678,7 @@ pub fn qsv_check_for_update(check_only: bool, no_confirm: bool) -> Result<bool,
     const GITHUB_RATELIMIT_MSG: &str =
         "Github is rate-limiting self-update checks at the moment. Try again in an hour.";
 
-    if env::var("QSV_NO_UPDATE").is_ok() {
+    if get_envvar_flag("QSV_NO_UPDATE") {
         return Ok(false);
     }
 
@@ -682,7 +715,14 @@ pub fn qsv_check_for_update(check_only: bool, no_confirm: bool) -> Result<bool,
     log::info!("Current version: {curr_version} Latest Release: {latest_release}");
 
     let mut updated = false;
-    if latest_release > &curr_version.to_string() {
+    let Ok(latest_release_sv) = semver::Version::parse(latest_release) else {
+        return fail!(format!("Can't parse latest release version: {latest_release}"));
+    };
+    let Ok(curr_version_sv) = semver::Version::parse(curr_version) else {
+        return fail!(format!("Can't parse current version: {curr_version}"));
+    };
+
+    if latest_release_sv > curr_version_sv {
         eprintln!("Update {latest_release} available. Current version is {curr_version}.");
         eprintln!(
             "Release notes: https://github.com/jqnatividad/qsv/releases/tag/{latest_release}\n"
@@ -1106,3 +1146,71 @@ pub fn transform(bs: &[u8], casei: bool) -> ByteString {
         bs.to_vec()
     }
 }
+
+pub fn load_dotenv() -> CliResult<()> {
+    // Use the default .env file in the current directory.
+    // If there is no .env file in the current directory, check if there is
+    // an .env file with the same filestem as the binary, in the same directory as the binary.
+    // If there is, use that. Failing that, qsv proceeds with its default settings and
+    // whatever manually set environment variables are present.
+
+    // check if there is an .env file in the current directory
+    if dotenvy::dotenv_override().is_ok() {
+        log::info!(
+            "Using .env file in current directory: {}",
+            std::env::current_dir()?.display()
+        );
+    } else {
+        // no .env file in the current directory or it was invalid
+        // now check if there is an .env file with the same name as the executable
+        // in the same directory as the executable
+        let qsv_binary_path = std::env::current_exe()?;
+
+        let qsv_dir = qsv_binary_path.parent().unwrap();
+
+        let qsv_binary_filestem = qsv_binary_path
+            .file_stem()
+            .unwrap()
+            .to_str()
+            .unwrap()
+            .to_string();
+
+        let mut qsv_binary_envprofile = qsv_dir.to_path_buf();
+        qsv_binary_envprofile.set_file_name(format!("{qsv_binary_filestem}.env"));
+
+        if std::path::Path::new(&qsv_binary_envprofile).exists() {
+            log::info!(
+                "Using binary .env file: {}",
+                qsv_binary_envprofile.display()
+            );
+            if let Err(e) = dotenvy::from_filename_override(qsv_binary_envprofile.clone()) {
+                return fail_clierror!(
+                    "Cannot process binary .env file - {}: {e}",
+                    qsv_binary_envprofile.display()
+                );
+            }
+        } else {
+            // there is no binary .env file, just use the default settings
+            // and whatever manually set environment variables are present
+            log::info!(
+                "No valid .env file found. Proceeding with default settings and current \
+                 environment variable settings."
+            );
+        }
+    }
+
+    Ok(())
+}
+
+pub fn get_envvar_flag(key: &str) -> bool {
+    if let Ok(tf_val) = std::env::var(key) {
+        let tf_val = tf_val.to_lowercase();
+        match tf_val {
+            s if s == "true" || s == "t" || s == "1" || s == "yes" || s == "y" => true,
+            s if s == "false" || s == "f" || s == "0" || s == "no" || s == "n" => false,
+            _ => false,
+        }
+    } else {
+        false
+    }
+}
