@@ -1,3 +1,5 @@
+use newline_converter::dos2unix;
+
 use crate::workdir::Workdir;
 
 fn data(headers: bool) -> Vec<Vec<String>> {
@@ -388,7 +390,7 @@ fn searchset_flag_complex() {
     let got_stderr: String = wrk.output_stderr(&mut cmd);
 
     let expected = wrk.load_test_resource("boston311-100-pii-searchset.csv");
-    assert_eq!(got, expected.replace("\r\n", "\n").trim_end());
+    assert_eq!(dos2unix(&got), dos2unix(&expected).trim_end());
 
     let expected_stderr = r#"{"rows_with_matches":5,"total_matches":6,"record_count":100}"#;
     assert_eq!(got_stderr.trim_end(), expected_stderr);
@@ -415,7 +417,7 @@ fn searchset_flag_complex_unmatched_output() {
     let got_stderr: String = wrk.output_stderr(&mut cmd);
 
     let expected = wrk.load_test_resource("boston311-100-pii-searchset.csv");
-    assert_eq!(got, expected.replace("\r\n", "\n").trim_end());
+    assert_eq!(dos2unix(&got), dos2unix(&expected).trim_end());
 
     let expected_stderr = r#"{"rows_with_matches":5,"total_matches":6,"record_count":100}"#;
     assert_eq!(got_stderr.trim_end(), expected_stderr);
