@@ -1,3 +1,5 @@
+use newline_converter::dos2unix;
+
 use crate::workdir::Workdir;
 
 static EXPECTED_CSV: &str = "\
@@ -23,7 +25,8 @@ fn sniff() {
 
     let got: String = wrk.stdout(&mut cmd);
 
-    let expected_end = r#"Delimiter: ,
+    let expected_end = dos2unix(
+        r#"Delimiter: ,
 Header Row: true
 Preamble Rows: 0
 Quote Char: none
@@ -39,9 +42,10 @@ Num Fields: 3
 Fields:
     0:  Text      h1
     1:  Unsigned  h2
-    2:  Text      h3"#;
+    2:  Text      h3"#,
+    );
 
-    assert!(got.ends_with(expected_end));
+    assert!(dos2unix(&got).trim_end().ends_with(expected_end.trim_end()));
 }
 
 #[test]
@@ -53,15 +57,7 @@ fn sniff_url() {
 
     let got: String = wrk.stdout(&mut cmd);
 
-    let expected_end = r#"Delimiter: ,
-Header Row: true
-Preamble Rows: 0
-Quote Char: "
-Flexible: false
-Is UTF8: true
-Retrieved Size (bytes): 47,702
-File Size (bytes): 47,702
-Sampled Records: 100
+    let expected_end = r#"Sampled Records: 100
 Estimated: true
 Num Records: 107
 Avg Record Len (bytes): 444
@@ -97,7 +93,7 @@ Fields:
     27:  Float     longitude
     28:  Text      source"#;
 
-    assert!(got.ends_with(expected_end.trim_end()));
+    assert!(dos2unix(&got).trim_end().ends_with(expected_end.trim_end()));
 }
 
 #[test]
@@ -109,15 +105,7 @@ fn sniff_url_snappy() {
 
     let got: String = wrk.stdout(&mut cmd);
 
-    let expected_end = r#"Delimiter: ,
-Header Row: true
-Preamble Rows: 0
-Quote Char: none
-Flexible: false
-Is UTF8: true
-Retrieved Size (bytes): 15,196
-File Size (bytes): 15,196
-Sampled Records: 100
+    let expected_end = r#"Sampled Records: 100
 Estimated: false
 Num Records: 100
 Avg Record Len (bytes): 444
@@ -153,7 +141,7 @@ Fields:
     27:  Float     longitude
     28:  Text      source"#;
 
-    assert!(got.ends_with(expected_end.trim_end()));
+    assert!(dos2unix(&got).trim_end().ends_with(expected_end.trim_end()));
 }
 
 #[test]
@@ -184,7 +172,7 @@ Fields:
     1:  Unsigned  h2
     2:  Text      h3"#;
 
-    assert!(got.ends_with(expected_end.trim_end()));
+    assert!(dos2unix(&got).trim_end().ends_with(expected_end.trim_end()));
 }
 
 #[test]
@@ -235,8 +223,8 @@ fn sniff_json() {
     cmd.arg("--json").arg(test_file);
 
     let got: String = wrk.stdout(&mut cmd);
+    let expected_end: &str = r#"ampled_records":3,"estimated":false,"num_records":3,"avg_record_len":10,"num_fields":4,"fields":["h1","h2","h3","h4"],"types":["Text","Unsigned","Text","Float"]}"#;
 
-    let expected_end = r#""delimiter_char":",","header_row":true,"preamble_rows":3,"quote_char":"none","flexible":false,"is_utf8":true,"retrieved_size":116,"file_size":116,"sampled_records":3,"estimated":false,"num_records":3,"avg_record_len":10,"num_fields":4,"fields":["h1","h2","h3","h4"],"types":["Text","Unsigned","Text","Float"]}"#;
     assert!(got.ends_with(expected_end));
 }
 
@@ -250,7 +238,8 @@ fn sniff_flexible_json() {
 
     let got: String = wrk.stdout(&mut cmd);
 
-    let expected_end = r#""delimiter_char":",","header_row":true,"preamble_rows":3,"quote_char":"none","flexible":true,"is_utf8":true,"retrieved_size":135,"file_size":135,"sampled_records":5,"estimated":false,"num_records":5,"avg_record_len":8,"num_fields":4,"fields":["h1","h2","h3","h4"],"types":["Text","Unsigned","Text","Float"]}"#;
+    let expected_end = r#"sampled_records":5,"estimated":false,"num_records":5,"avg_record_len":8,"num_fields":4,"fields":["h1","h2","h3","h4"],"types":["Text","Unsigned","Text","Float"]}"#;
+
     assert!(got.ends_with(expected_end));
 }
 
@@ -264,15 +253,8 @@ fn sniff_pretty_json() {
 
     let got: String = wrk.stdout(&mut cmd);
 
-    let expected_end = r#""delimiter_char": ",",
-  "header_row": true,
-  "preamble_rows": 3,
-  "quote_char": "none",
-  "flexible": false,
-  "is_utf8": true,
-  "retrieved_size": 116,
-  "file_size": 116,
-  "sampled_records": 3,
+    let expected_end = dos2unix(
+        r#"sampled_records": 3,
   "estimated": false,
   "num_records": 3,
   "avg_record_len": 10,
@@ -289,9 +271,10 @@ fn sniff_pretty_json() {
     "Text",
     "Float"
   ]
-}"#;
+}"#,
+    );
 
-    assert!(got.ends_with(expected_end));
+    assert!(dos2unix(&got).trim_end().ends_with(expected_end.trim_end()));
 }
 
 #[test]
@@ -307,15 +290,7 @@ fn sniff_sample() {
 
     let got: String = wrk.stdout(&mut cmd);
 
-    let expected_end = r#""delimiter_char": ",",
-  "header_row": true,
-  "preamble_rows": 0,
-  "quote_char": "none",
-  "flexible": false,
-  "is_utf8": true,
-  "retrieved_size": 9246,
-  "file_size": 9246,
-  "sampled_records": 7,
+    let expected_end = r#""sampled_records": 7,
   "estimated": false,
   "num_records": 15,
   "avg_record_len": 555,
@@ -390,7 +365,7 @@ fn sniff_sample() {
   ]
 }"#;
 
-    assert!(got.ends_with(expected_end));
+    assert!(dos2unix(&got).trim_end().ends_with(expected_end.trim_end()));
 }
 
 #[test]
@@ -403,15 +378,7 @@ fn sniff_prefer_dmy() {
 
     let got: String = wrk.stdout(&mut cmd);
 
-    let expected_end = r#"Delimiter: ,
-Header Row: true
-Preamble Rows: 0
-Quote Char: none
-Flexible: false
-Is UTF8: true
-Retrieved Size (bytes): 47,702
-File Size (bytes): 47,702
-Sampled Records: 100
+    let expected_end = r#"Sampled Records: 100
 Estimated: false
 Num Records: 100
 Avg Record Len (bytes): 444
@@ -447,7 +414,7 @@ Fields:
     27:  Float     longitude
     28:  Text      source"#;
 
-    assert!(got.ends_with(expected_end.trim_end()));
+    assert!(dos2unix(&got).trim_end().ends_with(expected_end.trim_end()));
 }
 
 #[test]
